﻿using Samba.Services;
using Samba.Services.Common;

using System.ComponentModel.Composition;

namespace BT.SambaServices.UpdateSettings
{
    [Export(typeof(IActionType))]
    public class UpdateProgramSettingsMsgSrv : ActionType
    {
        private readonly ISettingService _settingService;

        [ImportingConstructor]
        public UpdateProgramSettingsMsgSrv(ISettingService settingService)
        {
            _settingService = settingService;
        }

        public override void Process(ActionData actionData, RuleExecutionContext context)
        {
            var settings = actionData.GetAsString("SettingName");

            if (string.IsNullOrWhiteSpace(settings))
            {
                return;
            }

            var settingValue = actionData.GetAsString("SettingValue");
            var updateType = actionData.GetAsString("UpdateType");
            var isLocal = actionData.GetAsBoolean("IsLocal", false);
            var settingNames = settings.Split(',');
            var wasSettingUpdated = true;

            if (string.IsNullOrWhiteSpace(updateType))
            {
                updateType = "Update";
            }

            foreach (var settingName in settingNames)
            {
                var setting = settingName.Trim();

                if (string.IsNullOrWhiteSpace(setting))
                {
                    continue;
                }

                IProgramSetting programSetting;

                if (isLocal)
                {
                    programSetting = _settingService.ReadLocalSetting(settingName);
                }
                else
                {
                    programSetting = _settingService.ReadGlobalSetting(settingName);
                }

                switch (updateType)
                {
                    case "Increase":
                        if (int.TryParse(settingValue, out var value))
                        {
                            programSetting.IntegerValue += value;
                        }
                        break;

                    case "Decrease":
                        if (int.TryParse(settingValue, out var value2))
                        {
                            programSetting.IntegerValue -= value2;
                        }
                        break;

                    case "Delete":
                        _settingService.DeleteSetting(settingName, isLocal);
                        break;

                    case "Toggle":
                        wasSettingUpdated = false;
                        break;

                    case "Update":
                        programSetting.StringValue = settingValue;
                        break;

                    default:
                        wasSettingUpdated = false;
                        break;
                }

                if (!isLocal && wasSettingUpdated)
                {
                    _settingService.SaveProgramSettings();
                }
            }
        }

        protected override string GetActionKey()
        {
            return "UpdateProgramSettingMsgSrvCompat";
        }

        protected override string GetActionName()
        {
            return "Update Program Setting (Msg Srv Compat)";
        }

        protected override object GetDefaultData()
        {
            return new { SettingName = "", SettingValue = "", UpdateType = "Update", IsLocal = false };
        }
    }
}
